@@ -9,7 +9,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#ifdef HAVE_CRACK_H
 #include <crack.h>
+#endif
 #include <sys/types.h>
 #include <pwd.h>
 #include <unistd.h>
@@ -368,28 +370,28 @@ static int sequence(pwquality_settings_t *pwq, const char *new, void **auxerror)
 }
 
 static int
-usercheck(pwquality_settings_t *pwq, const char *new,
-          char *user)
+wordcheck(pwquality_settings_t *pwq, const char *new,
+          char *word)
 {
         char *f, *b;
-        int dist, userlen = strlen(user);
+        int dist, wordlen = strlen(word);
 
-        /* No point to check for username in password in 1-3 char
-         * usernames; it will be contained one way or another anyway. */
-        if (userlen < PWQ_MIN_WORD_LENGTH)
+        /* No point to check for word in password for 1-3 char
+         * words; it will be contained one way or another anyway. */
+        if (wordlen < PWQ_MIN_WORD_LENGTH)
                 return 0;
 
-        if (strstr(new, user) != NULL)
-                return 1;
+        if (strstr(new, word) != NULL)
+                return PWQ_ERROR_BAD_WORDS;
 
-        dist = distance(new, user);
+        dist = distance(new, word);
         if (dist >= 0 && dist < PWQ_DEFAULT_DIFF_OK)
-                return 1;
+                return PWQ_ERROR_BAD_WORDS;
 
-        /* now reverse the username, we can do that in place
+        /* now reverse the wordname, we can do that in place
                 as it is strdup-ed */
-        f = user;
-        b = user + userlen - 1;
+        f = word;
+        b = word + wordlen - 1;
         while (f < b) {
                 char c;
 
@@ -400,16 +402,50 @@ usercheck(pwquality_settings_t *pwq, const char *new,
                 ++f;
         }
 
-        if (strstr(new, user) != NULL)
-                return 1;
+        if (strstr(new, word) != NULL)
+                return PWQ_ERROR_BAD_WORDS;
 
-        dist = distance(new, user);
+        dist = distance(new, word);
         if (dist >= 0 && dist < PWQ_DEFAULT_DIFF_OK)
-                return 1;
+                return PWQ_ERROR_BAD_WORDS;
 
         return 0;
 }
 
+static int
+usercheck(pwquality_settings_t *pwq, const char *new,
+          char *user)
+{
+        int i, userlen;
+        int rv = 0;
+        char *subuser = calloc(pwq->user_substr+1, sizeof(char));
+
+        if (subuser == NULL) {
+                return PWQ_ERROR_MEM_ALLOC;
+        }
+
+        userlen = strlen(user);
+        if (pwq->user_substr >= PWQ_MIN_WORD_LENGTH &&
+            userlen > pwq->user_substr) {
+                for(i = 0; !rv && (i <= userlen - pwq->user_substr); i++) {
+                        strncpy(subuser, user+i, pwq->user_substr+1);
+                        subuser[pwq->user_substr] = '\0';
+                        rv = wordcheck(pwq, new, subuser);
+                }
+        }
+        else {
+                // if we already tested substrings, there's no need to test
+                // the whole username; all substrings would've been found :)
+                if (!rv)
+                        rv = wordcheck(pwq, new, user);
+        }
+        // translate wordcheck return
+        if (rv == PWQ_ERROR_BAD_WORDS)
+                rv = PWQ_ERROR_USER_CHECK;
+        free(subuser);
+        return rv;
+}
+
 static char *
 str_lower(char *string)
 {
@@ -445,7 +481,7 @@ wordlistcheck(pwquality_settings_t *pwq, const char *new,
 
                 if (strlen(p) >= PWQ_MIN_WORD_LENGTH) {
                         str_lower(p);
-                        if (usercheck(pwq, new, p)) {
+                        if (wordcheck(pwq, new, p)) {
                                 free(list);
                                 return PWQ_ERROR_BAD_WORDS;
                         }
@@ -466,7 +502,7 @@ gecoscheck(pwquality_settings_t *pwq, const char *new,
         struct passwd pwd;
         struct passwd *result;
         char *buf;
-        size_t bufsize;
+        long bufsize;
         int rv;
 
         bufsize = sysconf(_SC_GETPW_R_SIZE_MAX);
@@ -486,7 +522,7 @@ gecoscheck(pwquality_settings_t *pwq, const char *new,
         if (rv == PWQ_ERROR_BAD_WORDS)
                 rv = PWQ_ERROR_GECOS_CHECK;
 
-        free(buf);        
+        free(buf);
         return rv;
 }
 
@@ -557,9 +593,8 @@ password_check(pwquality_settings_t *pwq,
         if (!rv && sequence(pwq, new, auxerror))
                 rv = PWQ_ERROR_MAX_SEQUENCE;
 
-        if (!rv && usermono && pwq->user_check &&
-                usercheck(pwq, newmono, usermono))
-                rv = PWQ_ERROR_USER_CHECK;
+        if (!rv && usermono && pwq->user_check)
+                rv = usercheck(pwq, newmono, usermono);
 
         if (!rv && user && pwq->gecos_check)
                 rv = gecoscheck(pwq, newmono, user);
@@ -618,7 +653,7 @@ password_score(pwquality_settings_t *pwq, const char *password)
                                 buf[i] = abs(buf[i] - buf[i+1]);
                 }
 
-                for (i = 0; i < sizeof(freq); i++) {
+                for (i = 0; i < (int)sizeof(freq); i++) {
                         if (freq[i])
                                 ++score;
                 }
@@ -677,6 +712,7 @@ pwquality_check(pwquality_settings_t *pwq, const char *password,
         if (score != 0)
                 return score;
 
+        #ifdef HAVE_CRACK_H
         if (pwq->dict_check) {
                 msg = FascistCheck(password, pwq->dict_path);
                 if (msg) {
@@ -685,6 +721,7 @@ pwquality_check(pwquality_settings_t *pwq, const char *password,
                         return PWQ_ERROR_CRACKLIB_CHECK;
                 }
         }
+        #endif
 
         score = password_score(pwq, password);
 
