@@ -13,7 +13,9 @@
 #include <ctype.h>
 #include <errno.h>
 #include <dirent.h>
+#ifdef HAVE_CRACK_H
 #include <crack.h>
+#endif
 
 #include "pwquality.h"
 #include "pwqprivate.h"
@@ -36,6 +38,7 @@ pwquality_default_settings(void)
         pwq->oth_credit = PWQ_DEFAULT_OTH_CREDIT;
         pwq->dict_check = PWQ_DEFAULT_DICT_CHECK;
         pwq->user_check = PWQ_DEFAULT_USER_CHECK;
+        pwq->user_substr = PWQ_DEFAULT_USER_SUBSTR;
         pwq->enforcing = PWQ_DEFAULT_ENFORCING;
         pwq->retry_times = PWQ_DEFAULT_RETRY_TIMES;
         pwq->enforce_for_root = PWQ_DEFAULT_ENFORCE_ROOT;
@@ -70,6 +73,7 @@ static const struct setting_mapping s_map[] = {
  { "gecoscheck", PWQ_SETTING_GECOS_CHECK, PWQ_TYPE_INT},
  { "dictcheck", PWQ_SETTING_DICT_CHECK, PWQ_TYPE_INT},
  { "usercheck", PWQ_SETTING_USER_CHECK, PWQ_TYPE_INT},
+ { "usersubstr", PWQ_SETTING_USER_SUBSTR, PWQ_TYPE_INT},
  { "enforcing", PWQ_SETTING_ENFORCING, PWQ_TYPE_INT},
  { "badwords", PWQ_SETTING_BAD_WORDS, PWQ_TYPE_STR},
  { "dictpath", PWQ_SETTING_DICT_PATH, PWQ_TYPE_STR},
@@ -86,7 +90,7 @@ set_name_value(pwquality_settings_t *pwq, const char *name, const char *value)
         long val;
         char *endptr;
 
-        for (i = 0; i < sizeof(s_map)/sizeof(s_map[0]); i++) {
+        for (i = 0; i < (int)(sizeof(s_map)/sizeof(s_map[0])); i++) {
                 if (strcasecmp(s_map[i].name, name) == 0) {
                         switch(s_map[i].type) {
                         case PWQ_TYPE_INT:
@@ -349,6 +353,9 @@ pwquality_set_int_value(pwquality_settings_t *pwq, int setting, int value)
         case PWQ_SETTING_USER_CHECK:
                 pwq->user_check = value;
                 break;
+        case PWQ_SETTING_USER_SUBSTR:
+                pwq->user_substr = value;
+                break;
         case PWQ_SETTING_ENFORCING:
                 pwq->enforcing = value;
                 break;
@@ -471,10 +478,14 @@ pwquality_get_str_value(pwquality_settings_t *pwq, int setting, const char **val
                 *value = pwq->bad_words;
                 break;
         case PWQ_SETTING_DICT_PATH:
+                #ifdef HAVE_CRACK_H
                 if (pwq->dict_path)
                         *value = pwq->dict_path;
                 else
                         *value = GetDefaultCracklibDict();
+                #else
+                        *value = NULL;
+                #endif
                 break;
         default:
                 return PWQ_ERROR_NON_STR_SETTING;
